#ifndef __KRTCTYPEDEF_H__
#define __KRTCTYPEDEF_H__

#include <stdint.h>

#ifdef _WIN32
#ifdef KDV_RTC_SDK_EXPORTS
#define KDV_RTC_SDK_API __declspec(dllexport)
#else
#define KDV_RTC_SDK_API __declspec(dllimport)
#endif
#else
#ifdef KDV_RTC_SDK_EXPORTS
#define KDV_RTC_SDK_API __attribute__((visibility("default")))
#endif
#endif

#ifndef KDV_RTC_SDK_API
#define KDV_RTC_SDK_API
#endif

namespace KRTCSDK {
    /// @defgroup KRtcTypeDef 关键类型定义
    /// 关键类型定义
    /// @{
    enum KRTC_LOG_LEVEL
    {
        KRTC_LOG_LEVEL_ALL,
        KRTC_LOG_LEVEL_TRACE,
        KRTC_LOG_LEVEL_DEBUG,
        KRTC_LOG_LEVEL_INFO,
        KRTC_LOG_LEVEL_WARN,
        KRTC_LOG_LEVEL_ERROR,
        KRTC_LOG_LEVEL_FATAL,
        KRTC_LOG_LEVEL_OFF
    };

    enum VIDEO_TYPE
    {
        VIDEO_TYPE_RGBA8888,
    };

    typedef const char* userId;

    typedef void* winId;  // 窗口id，Android平台请将GLSurfaceView转为winId传入

    typedef void (*onLog)(const char* module, KRTC_LOG_LEVEL logLevel, const char* log);

    enum RTC_SDK_WARNING
    {
    };

    enum RTC_SDK_ERROR
    {
    };

    struct KRtcSdkInitParam
    {
        bool customVideoCapture;  ///< 启用或关闭自定义视频采集
        bool customVideoRender;   ///< 启用或关闭自定义视频渲染
        bool customAudioCapture;  ///< 启用或关闭自定义音频采集，启用后需用户手动调用sendRecordVideoData发送音频帧，发送间隔必须为20ms
        bool customAudioPlayout;  ///< 启用或关闭自定义音频输出
        bool customVideoEncode;   ///< 启用或关闭自定义视频编码
        bool customVideoDecode;   ///< 启用或关闭自定义视频解码
#if __ANDROID__
        void* jvm;
        void* jappContext;
#endif
    };

    struct KRtcSdkLogParam
    {
        bool        enableLog;
        const char* logPath;     ///< log路径，设nullptr表示不写入log文件
        onLog       callback;    ///< log回调，设nullptr不需要回调
    };

    struct LoginParam
    {
        const char* serverAddr;  ///< ipv4, ipv6, domain name
        userId      id;          ///< 用户id，匿名登录时填nullptr即可
        const char* token;       ///< token：appKey + userId + ExpiredTs + assignment
        const char* nickName;    ///< 用户别名(utf-8)
    };

    struct JoinRoomParam
    {
        const char* roomId;      ///< 房间id
        uint32_t    bitrate;     ///< 呼叫码率 0表示使用默认值，单位bps
        const char* roomPwd;     ///< 房间密码
    };

    struct CallIncomingParam
    {
        const char* roomId;
        const char* roomName;    ///< 房间别名
        uint32_t    bitrate;     ///< 房间码率
    };

    enum WINDOW_TYPE
    {
        WINDOW_TYPE_WINDOW,      ///< 应用窗口
        WINDOW_TYPE_SCREEN,      ///< 屏幕
    };

    struct ImageBuffer
    {
        const char* buffer;      ///< 图像内容（BGRA）
        uint32_t    length;      ///< 图像缓存大小
        uint32_t    width;       ///< 图像宽
        uint32_t    height;      ///< 图像高
    };

    struct ScreenList
    {
        int32_t   count;       ///< 屏幕个数
        uint32_t* screenId;    ///< 屏幕id
    };

    enum RESOLUTION
    {
        RESOLUTION_320x180,
        RESOLUTION_640x360,
        RESOLUTION_1280x720,
        RESOLUTION_480x270,
        RESOLUTION_960x540,
        RESOLUTION_1920x1080,
    };

    struct MainVideoStreamParam
    {
        uint8_t     layerCount;      ///< 同一条主流最多允许同时发送三层分辨率，双流仅允许发一层
        RESOLUTION  maxResolution;   ///< 最高层分辨率，中低层分辨率为 maxResolution/2，maxResolution/4
        uint8_t     maxFps;          ///< 最大帧率
        uint32_t    bitrate[3];      ///< 每层流的码率（bps），0 1 2 分别代表 低 中 高
        uint8_t     mediaIndex;      ///< 发布多条主流时每个视频源的索引号
    };

    struct DualVideoStreamParam
    {
        uint32_t    maxBitrate;      ///< 最大码率（bps）
        uint32_t    minBitrate;      ///< 最小码率（bps）
    };

    enum STREAM_TYPE
    {
        STREAM_TYPE_MAIN_VIDEO,      ///0< 主流视频
        STREAM_TYPE_DUAL_VIDEO,      ///1< 辅流视频
        STREAM_TYPE_AUDIO,           ///2< 音频
        STREAM_TYPE_VMP,             ///3< 画面合成码流
        STREAM_TYPE_MIX_AUDIO,       ///4< 混音流
    };

    struct SubscribeStreamParam
    {
        STREAM_TYPE type;
        uint8_t     mediaIndex;     ///< 多流时每个视频源的索引号，音频填0即可
        uint8_t     layerIndex;     ///< 订阅的分辨率层，辅流及音频填0即可
    };

    struct StreamParam
    {
        STREAM_TYPE type;
        uint8_t     mediaIndex;     ///< 发布多条辅流时每个视频源的索引号
        uint8_t     layerCount;     ///< 分辨率层数
        int32_t     width;          ///< 最大分辨率宽度
        int32_t     height;         ///< 最大分辨率高度
        uint8_t     maxFps;         ///< 最大帧率
        uint32_t    maxBitrate;     ///< 最大码率
    };

    // 视频帧数据（YUV420格式）
    struct VideoFrameData
    {
        int32_t  width;           ///< 帧宽
        int32_t  height;          ///< 帧高
        int32_t  strideY;         ///< Y跨距
        int32_t  strideU;         ///< U跨距
        int32_t  strideV;         ///< V跨距
        uint8_t* dataY;           ///< Y数据
        uint8_t* dataU;           ///< U数据
        uint8_t* dataV;           ///< V数据
        int64_t  timestampMs;     ///< 采集时间
    };

    struct AudioFrameData
    {
        uint8_t* data;             ///< 音频数据
        uint32_t dataSize;         ///< 数据长度
        uint32_t channelCount;     ///< 声道数
        uint32_t sampleRate;       ///< 采样频率
        uint32_t bitWidth;         ///< 采样位宽
    };

    enum H264_PROFILE
    {
        H264_PROFILE_BASELINE,
    };

    struct EncodedVideoData
    {
        uint8_t*     data;          ///< 已编码数据
        uint32_t     dataSize;      ///< 数据长度（字节数）
        uint32_t     width;         ///< 帧宽
        uint32_t     height;        ///< 帧高
        uint32_t     frameId;       ///< 帧ID
        int64_t      timestampMs;   ///< 采集/渲染时间（毫秒）
        bool         isKeyFrame;    ///< 是否关键帧（IDR帧）
        H264_PROFILE profile;	    ///< H264画质级别
    };

    struct VideoStatistics
    {
        uint32_t width;                ///< 视频宽度
        uint32_t height;               ///< 视频高度
        uint32_t frameRate;            ///< 视频帧率（fps）
        double   bitrate;              ///< 视频码率（bps）
        double   lossRate;             ///< 丢包率（%）
        uint32_t freezeCount;          ///< 视频卡顿帧数
        uint32_t totalFreezesDuration; ///< 视频卡顿时长
    };

    struct AudioStatistics
    {
        double bitrate;                ///< 音频码率（bps）
        double lossRate;               ///< 丢包率（%）
    };

    struct Statistics
    {
        uint32_t bytesSent;            ///< 累计发送（Byte）
        uint32_t bytesReceived;        ///< 累计接收（Byte）
        double   bitrateSend;          ///< 上行总码率（bps）
        double   bitrateRecv;          ///< 下行总码率（bps）
        double   lossRateSend;         ///< 上行总丢包率（％）
        double   lossRateRecv;         ///< 下行总丢包率（％）
    };

    enum DEVICETYPE
    {
        DEVICETYPE_MIC,
        DEVICETYPE_SPEAKER,
        DEVICETYPE_CAMERA,
    };

    struct DeviceInfo
    {
        const char* deviceName;
        const char* deviceId;
    };

    struct DeviceList
    {
        DEVICETYPE  type;
        int32_t     count;
        DeviceInfo* devices;
    };

    enum CAMERA_CAPTURE_ORIENTATION
    {
        CAMERA_CAPTURE_ORIENTATION_0,
        CAMERA_CAPTURE_ORIENTATION_90,
        CAMERA_CAPTURE_ORIENTATION_180,
        CAMERA_CAPTURE_ORIENTATION_270,
        CAMERA_CAPTURE_ORIENTATION_SYSTEM,
    };

    struct AudioLevelParam
    {
        userId id;
        int8_t level;  ///< 0~127
    };

    // 视频编码参数
    struct VideoCodecParam
    {
        uint32_t     width;	           ///< 帧宽
        uint32_t     height;           ///< 帧高
        uint32_t     bitrate;          ///< 码率，kbps
        uint32_t     frameRate;        ///< 帧率
        uint32_t     keyframeInterval; ///< 关键帧间隔
        H264_PROFILE profile;          ///< H264画质级别
    };

    struct AudioDeviceCapability
    {
        uint32_t channelCount;         ///< 声道数
        uint32_t sampleRate;           ///< 采样率
    };

    //采集原始视频数据
    typedef struct
    {
        uint8_t* data;			    ///< 采集数据
        uint32_t dataSize;		    ///< 数据长度（字节数）
        int32_t width;			    ///< 帧宽
        int32_t height;			    ///< 帧高
        VIDEO_TYPE videoType;	///< 视频类型
    } TRtcRawVideoData;

    typedef void (*onLocalVideoData)(const char* deviceId, VideoFrameData* data);

    typedef void (*onLocalAudioData)(const char* deviceId, AudioFrameData* data);

    typedef void (*onLocalAudioLevel)(const char* deviceId, int8_t level);


    /// @}
};

///
enum KRTC_CUSTOM_CALLBACK
{
    KRTC_CUSTOM_CALLBACK_NONE = 0,              ///< invalid backtype type
    KRTC_CUSTOM_CALLBACK_SIMPLECONFINFO = 1,    ///< data struct refer to simpleConfInfo_ind
    KRTC_CUSTOM_CALLBACK_DELAYCONFITIME,        ///< data struct refer to delayConfTime_ind
    KRTC_CUSTOM_CALLBACK_TERMSTATUSLIST_NOTIFY, ///< data struct refer to termStatusList_notify
    KRTC_CUSTOM_CALLBACK_CONFINFO_IND,          ///< data struct refer to confinfo_ind
    KRTC_CUSTOM_CALLBACK_CONFWILLEND_IND,       ///< data struct refer to confWillEnd_ind
    //KRTC_CUSTOM_CALLBACK_ONCONFPASSWORD_REQ,    ///< data struct refer to onConfPasswordReq
    KRTC_CUSTOM_CALLBACK_APPLYSPEAKER_RSP,      ///< data struct refer to applyspeaker_rsp
    KRTC_CUSTOM_CALLBACK_APPLYSPEADER_IND,      ///< data struct refer to applySpeaker_ind
    KRTC_CUSTOM_CALLBACK_SPECSPEAKER_RSP,       ///< data struct refer to specSpeaker_rsp
    KRTC_CUSTOM_CALLBACK_SPECSPEAKER_IND,       ///< data struct refer to specSpeaker_ind
    KRTC_CUSTOM_CALLBACK_INVITETERMINAL_ERROR,  ///< data struct refer to inviteTerminal_error
    KRTC_CUSTOM_CALLBACK_MAKETERMUTE_CMD,       ///< data struct refer to makeTerMute_cmd
    KRTC_CUSTOM_CALLBACK_TRANSPARENTMSG_IND,    ///< transparent msg indication
};

typedef void(*ONCUSTOMIZEDCALLBACK)(KRTC_CUSTOM_CALLBACK emEvent, const char* pszData, unsigned int dwSize, void* lpUser);


#endif
